@@ -1,76 +1,87 @@
 extern crate bincode;
 
-use bincode::{deserialize, deserialize_from, serialize};
-use std::fs;
-use std::io::prelude::*;
-use std::io::SeekFrom;
+use std::string::FromUtf8Error;
 
-pub enum OpenType {
-    OpenAndCreate,
-    Open,
-}
+use bincode::{deserialize, serialize};
 
-pub struct BinaryReader {
-    file: fs::File,
+pub mod filestream;
+
+pub struct BinaryReader<'a> {
+    stream: &'a mut dyn Stream,
 }
 
 #[derive(Debug)]
 pub enum BinaryError {
-    FSError(std::io::Error),
+    StreamError(StreamError),
     BinCodeErr(Box<bincode::ErrorKind>),
+    Utf8Error(FromUtf8Error),
 }
 
-impl BinaryReader {
-    pub fn new(filepath: &str, open_type: OpenType) -> Result<BinaryReader, BinaryError> {
-        let file;
+impl From<FromUtf8Error> for BinaryError {
+    fn from(error: FromUtf8Error) -> BinaryError {
+        BinaryError::Utf8Error(error)
+    }
+}
 
-        match open_type {
-            OpenType::OpenAndCreate => file = fs::File::create(filepath),
-            OpenType::Open => file = fs::File::open(filepath),
-        }
+impl From<StreamError> for BinaryError {
+    fn from(error: StreamError) -> BinaryError {
+        BinaryError::StreamError(error)
+    }
+}
 
-        match file {
-            Ok(f) => Ok(BinaryReader { file: f }),
-            Err(e) => Err(BinaryError::FSError(e)),
-        }
+#[derive(Debug)]
+pub enum StreamError {
+    OpenError,
+    WriteError,
+    ReadError,
+    SeekError,
+    TellError,
+}
+
+pub trait Stream {
+    fn write(&mut self, bytes: &Vec<u8>) -> Result<u64, StreamError>;
+    fn read(&mut self, buffer: &mut Vec<u8>) -> Result<u64, StreamError>;
+    fn seek(&mut self, from: u64) -> Result<u64, StreamError>;
+    fn tell(&mut self) -> Result<u64, StreamError>;
+}
+
+impl<'a> BinaryReader<'a> {
+    pub fn new(stream: &'a mut impl Stream) -> BinaryReader {
+        BinaryReader { stream }
     }
 
-    pub fn seek_to(&mut self, position: u64) -> Result<u64, BinaryError> {
-        let result = self.file.seek(SeekFrom::Start(position));
+    pub fn seek_to(&mut self, to: u64) -> Result<u64, BinaryError> {
+        let result = self.stream.seek(to);
 
         match result {
             Ok(r) => Ok(r),
-            Err(e) => Err(BinaryError::FSError(e)),
+            Err(e) => Err(BinaryError::StreamError(e)),
         }
     }
 
     pub fn get_cur_pos(&mut self) -> Result<u64, BinaryError> {
-        let result = self.file.seek(SeekFrom::Current(0));
+        let result = self.stream.tell();
 
         match result {
             Ok(r) => Ok(r),
-            Err(e) => Err(BinaryError::FSError(e)),
+            Err(e) => Err(BinaryError::StreamError(e)),
         }
     }
 
     pub fn read_string(&mut self) -> Result<String, BinaryError> {
-        let value = deserialize_from(&self.file);
+        let str_len = self.read_usize()?;
 
-        match value {
-            Ok(v) => Ok(v),
-            Err(e) => Err(BinaryError::BinCodeErr(e)),
-        }
+        let mut chars: Vec<u8> = vec![0; str_len];
+        self.stream.read(&mut chars)?;
+
+        let string = String::from_utf8(chars)?;
+        Ok(string)
     }
 
     pub fn read_f32(&mut self) -> Result<f32, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 4];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -83,12 +94,7 @@ impl BinaryReader {
     pub fn read_f64(&mut self) -> Result<f64, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 8];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -101,12 +107,7 @@ impl BinaryReader {
     pub fn read_isize(&mut self) -> Result<isize, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 8];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -119,12 +120,7 @@ impl BinaryReader {
     pub fn read_usize(&mut self) -> Result<usize, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 8];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -137,12 +133,7 @@ impl BinaryReader {
     pub fn read_u64(&mut self) -> Result<u64, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 8];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -155,12 +146,7 @@ impl BinaryReader {
     pub fn read_i64(&mut self) -> Result<i64, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 8];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -173,12 +159,7 @@ impl BinaryReader {
     pub fn read_u32(&mut self) -> Result<u32, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 4];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -191,12 +172,7 @@ impl BinaryReader {
     pub fn read_i32(&mut self) -> Result<i32, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 4];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -209,12 +185,7 @@ impl BinaryReader {
     pub fn read_u16(&mut self) -> Result<u16, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 2];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -227,12 +198,7 @@ impl BinaryReader {
     pub fn read_i16(&mut self) -> Result<i16, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 2];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -245,12 +211,7 @@ impl BinaryReader {
     pub fn read_u8(&mut self) -> Result<u8, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 1];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -263,12 +224,7 @@ impl BinaryReader {
     pub fn read_i8(&mut self) -> Result<i8, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; 1];
 
-        let read = self.file.read(&mut buffer);
-
-        match read {
-            Err(e) => return Err(BinaryError::FSError(e)),
-            _ => {}
-        };
+        self.stream.read(&mut buffer)?;
 
         let value = deserialize(&buffer);
 
@@ -280,63 +236,50 @@ impl BinaryReader {
 
     pub fn read_bytes(&mut self, length: u64) -> Result<Vec<u8>, BinaryError> {
         let mut buffer: Vec<u8> = vec![0; length as usize];
-        let bytes = self.file.read(&mut buffer);
+        let bytes = self.stream.read(&mut buffer);
 
         match bytes {
             Ok(_) => Ok(buffer),
-            Err(e) => Err(BinaryError::FSError(e)),
+            Err(e) => Err(BinaryError::StreamError(e)),
         }
     }
 }
 
-pub struct BinaryWriter {
-    file: fs::File,
+pub struct BinaryWriter<'a> {
+    stream: &'a mut dyn Stream,
 }
 
-impl BinaryWriter {
-    pub fn new(filepath: &str, open_type: OpenType) -> Result<BinaryWriter, BinaryError> {
-        let file;
-
-        match open_type {
-            OpenType::OpenAndCreate => file = fs::File::create(filepath),
-            OpenType::Open => file = fs::File::open(filepath),
-        }
-
-        match file {
-            Ok(f) => Ok(BinaryWriter { file: f }),
-            Err(e) => Err(BinaryError::FSError(e)),
-        }
+impl<'a> BinaryWriter<'a> {
+    pub fn new(stream: &'a mut impl Stream) -> BinaryWriter {
+        BinaryWriter { stream }
     }
 
-    pub fn seek_to(&mut self, position: u64) -> Result<u64, BinaryError> {
-        let result = self.file.seek(SeekFrom::Start(position));
+    pub fn seek_to(&mut self, to: u64) -> Result<u64, BinaryError> {
+        let result = self.stream.seek(to);
 
         match result {
-            Ok(seek) => Ok(seek),
-            Err(e) => Err(BinaryError::FSError(e)),
+            Ok(r) => Ok(r),
+            Err(e) => Err(BinaryError::StreamError(e)),
         }
     }
 
     pub fn get_cur_pos(&mut self) -> Result<u64, BinaryError> {
-        let result = self.file.seek(SeekFrom::Current(0));
+        let result = self.stream.tell();
 
         match result {
             Ok(pos) => Ok(pos),
-            Err(e) => Err(BinaryError::FSError(e)),
+            Err(e) => Err(BinaryError::StreamError(e)),
         }
     }
 
     pub fn write_string(&mut self, value: String) -> Option<BinaryError> {
-        let data = serialize(&value);
+        let bytes = value.as_bytes();
 
-        let data = match data {
-            Ok(d) => d,
-            Err(e) => return Some(BinaryError::BinCodeErr(e)),
-        };
-        let result = self.file.write(&data);
+        self.write_usize(bytes.len());
+        let result = self.stream.write(&bytes.to_vec());
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -348,10 +291,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -363,10 +306,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -378,10 +321,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -393,10 +336,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -408,10 +351,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -423,10 +366,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -438,10 +381,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -453,10 +396,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -468,10 +411,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -483,10 +426,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -498,10 +441,10 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
@@ -513,19 +456,19 @@ impl BinaryWriter {
             Ok(d) => d,
             Err(e) => return Some(BinaryError::BinCodeErr(e)),
         };
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
 
     pub fn write_bytes(&mut self, data: Vec<u8>) -> Option<BinaryError> {
-        let result = self.file.write(&data);
+        let result = self.stream.write(&data);
 
         match result {
-            Err(e) => Some(BinaryError::FSError(e)),
+            Err(e) => Some(BinaryError::StreamError(e)),
             _ => None,
         }
     }
